Django=={{ dj_version }}
django-rainbowtests==0.5.1
gunicorn==19.4.5
pep8==1.7.0
pylama==7.0.7{% if POSTGRES %}
psycopg2==2.6.1{% endif %}{% if CELERY %}
Celery==3.1.23{% endif %}{% if REDIS %}
redis==2.10.5{% endif %}
{# keep trailing newline #}
