# Local settings

# Turn debug mode on
DEBUG = True
TEMPLATE_DEBUG = True
{% if POSTGRES %}
# Database connection
DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.postgresql_psycopg2',
        'NAME': '{{ proj_name }}',
        'USER': '{{ db_user }}',
        'PASSWORD': '{{ db_pass }}',
        'HOST': '127.0.0.1',
    }
}
{% endif %}

# # email settings
# # To create debug email server for your awesome project visit
# # https://debugmail.io/ it's easy and nice service for debug your emails
# EMAIL_HOST = 'debugmail.io'
# EMAIL_PORT = 25
# EMAIL_HOST_USER = '<user login>'
# EMAIL_HOST_PASSWORD = '<user password>'
# EMAIL_BACKEND = 'django.core.mail.backends.smtp.EmailBackend'

# ADMINS = (
#     ('<admin name>', '<admin email>'),
# )
{# keep trailing newline #}
